// Dynamic link library implementation of a typed input source

#include "NSDLL.h" 

/***************************/
/* Activation of component */
__declspec(dllexport) void performInput(
	DLLData	*instance,	// Pointer to instance data (may be NULL)
	NSFloat	*data, 		// Pointer to the data
	int rows, 		// Number of rows of data
	int cols 		// Number of cols of data
	)
{
	int i;
	NSFloat *myData = getUserData(instance);
	if (myData)
		for (i=0; i<rows*cols; i++)
			data[i] = myData[i];
}

/******************************************/
/* Management of instance data (OPTIONAL) */
__declspec(dllexport) DLLData *allocInput(
	DLLData	*oldInstance,	// Pointer to the last instance if reallocating
	int rows, 		// Number of rows of data
	int cols 		// Number of cols of data
	)
{
	int row,col;

	DLLData *instance = allocDLLInstance(oldInstance);
	NSFloat *myData = (NSFloat *)calloc(rows*cols, sizeof(NSFloat));
	setUserData(instance, myData);
	setParameterName(instance, 0, 0, "Row", FALSE);
	setIntParameter(instance, 0, 0, 0, FALSE);
	setParameterName(instance, 0, 1, "Col", FALSE);
	setIntParameter(instance, 0, 1, 0, FALSE);
	setParameterName(instance, 1, 0, "Value", TRUE);
	setFloatParameter(instance, 1, 0, 0.0f, FALSE);

	row = getIntParameter(instance, 0, 0);
	col = getIntParameter(instance, 0, 1);
	if ( row<=rows && col<=cols )
		myData[col + row*cols] = getFloatParameter(instance, 1, 0);

	return instance;
}

__declspec(dllexport) void freeInput(DLLData *instance)
{
	NSFloat *myData = getUserData(instance);
	if (myData)
		free(myData);
	freeDLLInstance(instance);
}
